@@ -35,21 +35,16 @@
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
 
+#include <common/na-iprefs.h>
+
 #include "nact-application.h"
 #include "nact-iactions-list.h"
+#include "nact-tree-model.h"
 
 /* private interface data
  */
 struct NactIActionsListInterfacePrivate {
-};
-
-/* column ordering
- */
-enum {
-	IACTIONS_LIST_ICON_COLUMN = 0,
-	IACTIONS_LIST_LABEL_COLUMN,
-	IACTIONS_LIST_NAOBJECT_COLUMN,
-	IACTIONS_LIST_N_COLUMN
+	void *empty;						/* so that gcc -pedantic is happy */
 };
 
 /* data set against GObject
@@ -58,13 +53,13 @@ enum {
 #define ACCEPT_MULTIPLE_SELECTION		"iactions-list-accept-multiple-selection"
 #define IS_FILLING_LIST					"iactions-list-is-filling-list"
 #define SEND_SELECTION_CHANGED_MESSAGE	"iactions-list-send-selection-changed-message"
+#define HAVE_DND_MODE					"iactions-list-dnd-mode"
 
 static GType      register_type( void );
 static void       interface_base_init( NactIActionsListInterface *klass );
 static void       interface_base_finalize( NactIActionsListInterface *klass );
 
 static GSList    *v_get_actions( NactWindow *window );
-static void       v_on_selection_changed( GtkTreeSelection *selection, gpointer user_data );
 static gboolean   v_on_button_press_event( GtkWidget *widget, GdkEventButton *event, gpointer data );
 static gboolean   v_on_key_pressed_event( GtkWidget *widget, GdkEventKey *event, gpointer data );
 static gboolean   v_is_modified_action( NactWindow *window, const NAAction *action );
@@ -72,11 +67,10 @@ static gboolean   v_is_valid_action( NactWindow *window, const NAAction *action
 static gboolean   v_is_modified_profile( NactWindow *window, const NAActionProfile *profile );
 static gboolean   v_is_valid_profile( NactWindow *window, const NAActionProfile *profile );
 
+static void       on_selection_changed( GtkTreeSelection *selection, NactIActionsList *instance );
 static void       display_label( GtkTreeViewColumn *column, GtkCellRenderer *cell, GtkTreeModel *model, GtkTreeIter *iter, NactWindow *window );
 static void       setup_action( GtkWidget *treeview, GtkTreeStore *model, GtkTreeIter *iter, NAAction *action );
 static void       setup_profile( GtkWidget *treeview, GtkTreeStore *model, GtkTreeIter *iter, NAActionProfile *profile );
-static gint       sort_actions_list( GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, NactWindow *window );
-static gboolean   filter_visible( GtkTreeModel *model, GtkTreeIter *iter, gpointer data );
 static GtkWidget *get_actions_list_widget( NactWindow *window );
 static GSList    *get_expanded_rows( NactWindow *window );
 static void       expand_rows( NactWindow *window, GSList *expanded );
@@ -99,7 +93,7 @@ static GType
 register_type( void )
 {
 	static const gchar *thisfn = "nact_iactions_list_register_type";
-	g_debug( "%s", thisfn );
+	GType type;
 
 	static const GTypeInfo info = {
 		sizeof( NactIActionsListInterface ),
@@ -113,7 +107,9 @@ register_type( void )
 		NULL
 	};
 
-	GType type = g_type_register_static( G_TYPE_INTERFACE, "NactIActionsList", &info, 0 );
+	g_debug( "%s", thisfn );
+
+	type = g_type_register_static( G_TYPE_INTERFACE, "NactIActionsList", &info, 0 );
 
 	g_type_interface_add_prerequisite( type, G_TYPE_OBJECT );
 
@@ -127,7 +123,7 @@ interface_base_init( NactIActionsListInterface *klass )
 	static gboolean initialized = FALSE;
 
 	if( !initialized ){
-		g_debug( "%s: klass=%p", thisfn, klass );
+		g_debug( "%s: klass=%p", thisfn, ( void * ) klass );
 
 		klass->private = g_new0( NactIActionsListInterfacePrivate, 1 );
 
@@ -142,7 +138,7 @@ interface_base_finalize( NactIActionsListInterface *klass )
 	static gboolean finalized = FALSE ;
 
 	if( !finalized ){
-		g_debug( "%s: klass=%p", thisfn, klass );
+		g_debug( "%s: klass=%p", thisfn, ( void * ) klass );
 
 		g_free( klass->private );
 
@@ -152,50 +148,42 @@ interface_base_finalize( NactIActionsListInterface *klass )
 
 /**
  * Allocates and initializes the ActionsList widget.
+ *
+ * GtkTreeView is created with NactTreeModel model
+ * NactTreeModel
+ *   implements EggTreeMultiDragSourceIface
+ *   is derived from GtkTreeModelFilter
+ *     GtkTreeModelFilter is built on top of GtkTreeStore
  */
 void
 nact_iactions_list_initial_load( NactWindow *window )
 {
 	static const gchar *thisfn = "nact_iactions_list_initial_load";
-	g_debug( "%s: window=%p", thisfn, window );
+	GtkWidget *widget, *label;
+	NactTreeModel *model;
+	GtkTreeViewColumn *column;
+	GtkCellRenderer *renderer;
 
+	g_debug( "%s: window=%p", thisfn, ( void * ) window );
 	g_assert( NACT_IS_IACTIONS_LIST( window ));
 	g_assert( NACT_IS_WINDOW( window ));
 
-	GtkWidget *widget = get_actions_list_widget( window );
+	widget = get_actions_list_widget( window );
 
 	/* associates the ActionsList to the label */
-	GtkWidget *label = base_window_get_widget( BASE_WINDOW( window ), "ActionsListLabel" );
+	label = base_window_get_widget( BASE_WINDOW( window ), "ActionsListLabel" );
 	gtk_label_set_mnemonic_widget( GTK_LABEL( label ), widget );
 
 	nact_iactions_list_set_send_selection_changed_on_fill_list( window, FALSE );
 	nact_iactions_list_set_is_filling_list( window, FALSE );
 
-	/* create the model */
-	GtkTreeStore *ts_model = gtk_tree_store_new(
-			IACTIONS_LIST_N_COLUMN, GDK_TYPE_PIXBUF, G_TYPE_STRING, NA_OBJECT_TYPE );
-
-	gtk_tree_sortable_set_default_sort_func(
-			GTK_TREE_SORTABLE( ts_model ),
-	        ( GtkTreeIterCompareFunc ) sort_actions_list, window, NULL );
-
-	gtk_tree_sortable_set_sort_column_id(
-			GTK_TREE_SORTABLE( ts_model ),
-			IACTIONS_LIST_LABEL_COLUMN, GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID );
-
-	GtkTreeModel *tmf_model = gtk_tree_model_filter_new( GTK_TREE_MODEL( ts_model ), NULL );
-
-	gtk_tree_model_filter_set_visible_func(
-			GTK_TREE_MODEL_FILTER( tmf_model ), ( GtkTreeModelFilterVisibleFunc ) filter_visible, window, NULL );
-
-	gtk_tree_view_set_model( GTK_TREE_VIEW( widget ), tmf_model );
+	model = nact_tree_model_new( window );
+	gtk_tree_view_set_model( GTK_TREE_VIEW( widget ), GTK_TREE_MODEL( model ));
 	gtk_tree_view_set_enable_tree_lines( GTK_TREE_VIEW( widget ), TRUE );
-
-	g_object_unref( tmf_model );
-	g_object_unref( ts_model );
+	g_object_unref( model );
 
 	/* create visible columns on the tree view */
-	GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes(
+	column = gtk_tree_view_column_new_with_attributes(
 			"icon", gtk_cell_renderer_pixbuf_new(), "pixbuf", IACTIONS_LIST_ICON_COLUMN, NULL );
 	gtk_tree_view_append_column( GTK_TREE_VIEW( widget ), column );
 
@@ -204,7 +192,7 @@ nact_iactions_list_initial_load( NactWindow *window )
 	column = gtk_tree_view_column_new();
 	gtk_tree_view_column_set_title( column, "label" );
 	gtk_tree_view_column_set_sort_column_id( column, IACTIONS_LIST_LABEL_COLUMN );
-	GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
+	renderer = gtk_cell_renderer_text_new();
 	gtk_tree_view_column_pack_start( column, renderer, TRUE );
 	gtk_tree_view_column_set_cell_data_func(
 			column, renderer, ( GtkTreeCellDataFunc ) display_label, window, NULL );
@@ -218,22 +206,25 @@ void
 nact_iactions_list_runtime_init( NactWindow *window )
 {
 	static const gchar *thisfn = "nact_iactions_list_runtime_init";
-	g_debug( "%s: window=%p", thisfn, window );
+	GtkWidget *widget;
+	GtkTreeSelection *selection;
 
+	g_debug( "%s: window=%p", thisfn, ( void * ) window );
 	g_assert( NACT_IS_IACTIONS_LIST( window ));
 	g_assert( NACT_IS_WINDOW( window ));
 
-	GtkWidget *widget = get_actions_list_widget( window );
+	widget = get_actions_list_widget( window );
 	g_assert( GTK_IS_WIDGET( widget ));
 
 	nact_iactions_list_fill( window, TRUE );
+	nact_tree_model_runtime_init_dnd( window, GTK_TREE_VIEW( widget ));
 
-	/* set up selection */
+	/* set up selection control */
 	nact_window_signal_connect(
 			window,
 			G_OBJECT( gtk_tree_view_get_selection( GTK_TREE_VIEW( widget ))),
 			"changed",
-			G_CALLBACK( v_on_selection_changed ));
+			G_CALLBACK( on_selection_changed ));
 
 	/* catch press 'Enter' */
 	nact_window_signal_connect(
@@ -250,7 +241,7 @@ nact_iactions_list_runtime_init( NactWindow *window )
 			G_CALLBACK( v_on_button_press_event ));
 
 	/* clear the selection */
-	GtkTreeSelection *selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( widget ));
+	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( widget ));
 	gtk_tree_selection_unselect_all( selection );
 }
 
@@ -261,40 +252,46 @@ void
 nact_iactions_list_fill( NactWindow *window, gboolean keep_expanded )
 {
 	static const gchar *thisfn = "nact_iactions_list_fill";
-	g_debug( "%s: window=%p", thisfn, window );
+	GSList *expanded = NULL;
+	GtkWidget *widget;
+	GtkTreeModelFilter *tmf_model;
+	GtkTreeStore *ts_model;
+	GSList *actions, *ia;
+	GSList *profiles, *ip;
+	GtkTreeIter iter, profile_iter;
+	NAAction *action;
+	NAActionProfile *profile;
+
+	g_debug( "%s: window=%p, keep_expanded=%s",
+			thisfn, ( void * ) window, keep_expanded ? "True":"False" );
 
 	g_assert( NACT_IS_IACTIONS_LIST( window ));
 
 	nact_iactions_list_set_is_filling_list( window, TRUE );
 
-	GSList *expanded = NULL;
 	if( keep_expanded ){
 		expanded = get_expanded_rows( window );
 	}
 
-	GtkWidget *widget = get_actions_list_widget( window );
-	GtkTreeModelFilter *tmf_model = GTK_TREE_MODEL_FILTER( gtk_tree_view_get_model( GTK_TREE_VIEW( widget )));
-	GtkTreeStore *ts_model = GTK_TREE_STORE( gtk_tree_model_filter_get_model( tmf_model ));
+	widget = get_actions_list_widget( window );
+	tmf_model = GTK_TREE_MODEL_FILTER( gtk_tree_view_get_model( GTK_TREE_VIEW( widget )));
+	ts_model = GTK_TREE_STORE( gtk_tree_model_filter_get_model( tmf_model ));
 	gtk_tree_store_clear( ts_model );
 
-	GSList *actions = v_get_actions( window );
-	GSList *ia;
+	actions = v_get_actions( window );
 	g_debug( "%s: actions has %d elements", thisfn, g_slist_length( actions ));
 
 	for( ia = actions ; ia != NULL ; ia = ia->next ){
-		GtkTreeIter iter;
-		NAAction *action = NA_ACTION( ia->data );
+		action = NA_ACTION( ia->data );
 		gtk_tree_store_append( ts_model, &iter, NULL );
 		gtk_tree_store_set( ts_model, &iter, IACTIONS_LIST_NAOBJECT_COLUMN, action, -1);
 		setup_action( widget, ts_model, &iter, action );
-		g_debug( "%s: action=%p", thisfn, action );
+		g_debug( "%s: action=%p", thisfn, ( void * ) action );
 
 		if( is_edition_mode( window )){
-			GSList *profiles = na_action_get_profiles( action );
-			GSList *ip;
-			GtkTreeIter profile_iter;
+			profiles = na_action_get_profiles( action );
 			for( ip = profiles ; ip ; ip = ip->next ){
-				NAActionProfile *profile = NA_ACTION_PROFILE( ip->data );
+				profile = NA_ACTION_PROFILE( ip->data );
 				gtk_tree_store_append( ts_model, &profile_iter, &iter );
 				gtk_tree_store_set( ts_model, &profile_iter, IACTIONS_LIST_NAOBJECT_COLUMN, profile, -1 );
 				setup_profile( widget, ts_model, &profile_iter, profile );
@@ -328,6 +325,9 @@ nact_iactions_list_fill( NactWindow *window, gboolean keep_expanded )
  * - whose uuid is the requested uuid
  * - or whose label is the most close of the required label (if uuid is not found)
  *
+ * if label is NULL, then we consider that the action must be found, and we
+ * explore the whole list for the uuid ; if not found, we select the last item.
+ *
  * if we want select a profile
  * - set type = NA_ACTION_PROFILE_TYPE
  * - set uuid = uuid of the parent action
@@ -347,61 +347,68 @@ void
 nact_iactions_list_set_selection( NactWindow *window, GType type, const gchar *uuid, const gchar *label )
 {
 	static const gchar *thisfn = "nact_iactions_list_set_selection";
-	g_debug( "%s: window=%p, type=%s, uuid=%s, label=%s", thisfn, window, type == NA_ACTION_TYPE ? "action":"profile", uuid, label );
-
-	GtkWidget *list = get_actions_list_widget( window );
-	GtkTreeSelection *selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( list ));
-
-	if( !uuid || !strlen( uuid )){
-		g_debug( "%s: null or empty uuid: unselect all", thisfn );
-		gtk_tree_selection_unselect_all( selection );
-		v_on_selection_changed( selection, window );
-		return;
-	}
-
+	GtkWidget *list;
+	GtkTreeSelection *selection;
 	GtkTreeIter iter;
-	GtkTreeModel *model = gtk_tree_view_get_model( GTK_TREE_VIEW( list ));
-	gboolean iterok = gtk_tree_model_get_iter_first( model, &iter );
-	if( !iterok ){
-		g_debug( "%s: empty actions list: unselect all", thisfn );
-		gtk_tree_selection_unselect_all( selection );
-		v_on_selection_changed( selection, window );
-		return;
-	}
-
+	GtkTreeModel *model;
+	gboolean iterok, iter_child_ok;
 	gboolean found = FALSE;
 	GtkTreeIter previous;
 	NAObject *iter_object;
 	gchar *iter_uuid, *iter_label;
+	gint ret_uuid, ret_label;
+	guint nb_profiles;
+	GtkTreeIter iter_child, previous_child;
+
+	g_debug( "%s: window=%p, type=%s, uuid=%s, label=%s",
+			thisfn, ( void * ) window, type == NA_ACTION_TYPE ? "action":"profile", uuid, label );
+
+	list = get_actions_list_widget( window );
+	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( list ));
+
+	if( !uuid || !strlen( uuid )){
+		g_debug( "%s: null or empty uuid: unselect all", thisfn );
+		gtk_tree_selection_unselect_all( selection );
+		on_selection_changed( selection, NACT_IACTIONS_LIST( window ));
+		return;
+	}
+
+	model = gtk_tree_view_get_model( GTK_TREE_VIEW( list ));
+	iterok = gtk_tree_model_get_iter_first( model, &iter );
+	if( !iterok ){
+		g_debug( "%s: empty actions list: unselect all", thisfn );
+		gtk_tree_selection_unselect_all( selection );
+		on_selection_changed( selection, NACT_IACTIONS_LIST( window ));
+		return;
+	}
 
 	while( iterok && !found ){
 		gtk_tree_model_get( model, &iter, IACTIONS_LIST_NAOBJECT_COLUMN, &iter_object, IACTIONS_LIST_LABEL_COLUMN, &iter_label, -1 );
 		g_assert( NA_IS_ACTION( iter_object ));
 
 		iter_uuid = na_action_get_uuid( NA_ACTION( iter_object ));
-		gint ret_uuid = g_ascii_strcasecmp( iter_uuid, uuid );
-		guint nb_profiles = na_action_get_profiles_count( NA_ACTION( iter_object ));
+		ret_uuid = g_ascii_strcasecmp( iter_uuid, uuid );
+		nb_profiles = na_action_get_profiles_count( NA_ACTION( iter_object ));
 
 		if( type == NA_ACTION_TYPE || ( ret_uuid == 0 && nb_profiles == 1 )){
-			gint ret_label = g_utf8_collate( iter_label, label );
+			ret_label = label ? g_utf8_collate( iter_label, label ) : -1;
 
 			if( ret_uuid == 0 || ret_label > 0 ){
-				g_debug( "%s: selecting action iter_object=%p, ret_uuid=%d, ret_label=%d", thisfn, iter_object, ret_uuid, ret_label );
+				g_debug( "%s: selecting action iter_object=%p, ret_uuid=%d, ret_label=%d", thisfn, ( void * ) iter_object, ret_uuid, ret_label );
 				gtk_tree_selection_select_iter( selection, &iter );
 				found = TRUE;
 				break;
 			}
 
 		} else if( ret_uuid == 0 && gtk_tree_model_iter_has_child( model, &iter )){
-			GtkTreeIter iter_child, previous_child;
-			gboolean iter_child_ok = gtk_tree_model_iter_children( model, &iter_child, &iter );
+			iter_child_ok = gtk_tree_model_iter_children( model, &iter_child, &iter );
 
 			while( iter_child_ok ){
 				gtk_tree_model_get( model, &iter_child, IACTIONS_LIST_NAOBJECT_COLUMN, &iter_object, IACTIONS_LIST_LABEL_COLUMN, &iter_label, -1 );
-				gint ret_label = g_utf8_collate( iter_label, label );
+				ret_label = g_utf8_collate( iter_label, label );
 
 				if( ret_label >= 0 ){
-					g_debug( "%s: selecting profile iter_object=%p, ret_uuid=%d, ret_label=%d", thisfn, iter_object, ret_uuid, ret_label );
+					g_debug( "%s: selecting profile iter_object=%p, ret_uuid=%d, ret_label=%d", thisfn, ( void * ) iter_object, ret_uuid, ret_label );
 					gtk_tree_selection_select_iter( selection, &iter_child );
 					found = TRUE;
 					break;
@@ -430,18 +437,23 @@ void
 nact_iactions_list_select_first( NactWindow *window )
 {
 	static const gchar *thisfn = "nact_iactions_list_select_first";
-	g_debug( "%s: window=%p", thisfn, window );
-
-	GtkWidget *list = get_actions_list_widget( window );
-	GtkTreeSelection *selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( list ));
-
+	GtkWidget *list;
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
 	GtkTreeIter iter;
-	GtkTreeModel *model = gtk_tree_view_get_model( GTK_TREE_VIEW( list ));
-	gboolean iterok = gtk_tree_model_get_iter_first( model, &iter );
+	gboolean iterok;
+
+	g_debug( "%s: window=%p", thisfn, ( void * ) window );
+
+	list = get_actions_list_widget( window );
+	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( list ));
+
+	model = gtk_tree_view_get_model( GTK_TREE_VIEW( list ));
+	iterok = gtk_tree_model_get_iter_first( model, &iter );
 	if( !iterok ){
 		g_debug( "%s: empty actions list: unselect all", thisfn );
 		gtk_tree_selection_unselect_all( selection );
-		v_on_selection_changed( selection, window );
+		on_selection_changed( selection, NACT_IACTIONS_LIST( window ));
 		return;
 	}
 
@@ -460,17 +472,20 @@ nact_iactions_list_set_focus( NactWindow *window )
 
 /**
  * Returns the currently selected action or profile.
+ * TODO: remove this function
  */
 NAObject *
 nact_iactions_list_get_selected_object( NactWindow *window )
 {
 	NAObject *object = NULL;
-
-	GtkWidget *treeview = get_actions_list_widget( window );
-	GtkTreeSelection *selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( treeview ));
-
+	GtkWidget *treeview;
+	GtkTreeSelection *selection;
 	GtkTreeModel *tm_model;
 	GtkTreeIter iter;
+
+	treeview = get_actions_list_widget( window );
+	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( treeview ));
+
 	if( gtk_tree_selection_get_selected( selection, &tm_model, &iter )){
 
 		gtk_tree_model_get( tm_model, &iter, IACTIONS_LIST_NAOBJECT_COLUMN, &object, -1 );
@@ -487,33 +502,47 @@ nact_iactions_list_get_selected_object( NactWindow *window )
  *
  * The returned GSList should be freed by the caller (g_slist_free),
  * without freing not unref any of the contained objects.
+ * TODO: remove this function
  */
 GSList *
 nact_iactions_list_get_selected_actions( NactWindow *window )
 {
-	GSList *actions = NULL;
-
-	GtkWidget *treeview = get_actions_list_widget( window );
-	GtkTreeSelection *selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( treeview ));
+	return( nact_iactions_list_get_selected_items( NACT_IACTIONS_LIST( window )));
+}
 
+/**
+ * Returns the currently selected actions when in export mode.
+ *
+ * The returned GSList should be g_slist_free() by the caller,
+ * without freing nor unref any of the contained objects.
+ */
+GSList *
+nact_iactions_list_get_selected_items( NactIActionsList *instance )
+{
+	GSList *items = NULL;
+	GtkWidget *treeview;
+	GtkTreeSelection *selection;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	GList *it;
-	NAAction *action;
+	GList *it, *listrows;
+	NAObject *object;
+	GtkTreePath *path;
+
+	treeview = get_actions_list_widget( NACT_WINDOW( instance ));
+	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( treeview ));
+	listrows = gtk_tree_selection_get_selected_rows( selection, &model );
 
-	GList *listrows = gtk_tree_selection_get_selected_rows( selection, &model );
 	for( it = listrows ; it ; it = it->next ){
-		GtkTreePath *path = ( GtkTreePath * ) it->data;
+		path = ( GtkTreePath * ) it->data;
 		gtk_tree_model_get_iter( model, &iter, path );
-
-		gtk_tree_model_get( model, &iter, IACTIONS_LIST_NAOBJECT_COLUMN, &action, -1 );
-		actions = g_slist_prepend( actions, action );
+		gtk_tree_model_get( model, &iter, IACTIONS_LIST_NAOBJECT_COLUMN, &object, -1 );
+		items = g_slist_prepend( items, object );
 	}
 
 	g_list_foreach( listrows, ( GFunc ) gtk_tree_path_free, NULL );
 	g_list_free( listrows );
 
-	return( actions );
+	return( g_slist_reverse( items ));
 }
 
 /*void
@@ -555,6 +584,8 @@ void
 nact_iactions_list_toggle_collapse( NactWindow *window, const NAAction *action )
 {
 	static const gchar *thisfn = "nact_iactions_list_toggle_collapse";
+	NAObject *iter_object;
+	GtkTreePath *path;
 
 	GtkWidget *treeview = get_actions_list_widget( window );
 	GtkTreeModel *model = gtk_tree_view_get_model( GTK_TREE_VIEW( treeview ));
@@ -564,21 +595,20 @@ nact_iactions_list_toggle_collapse( NactWindow *window, const NAAction *action )
 
 	while( iterok ){
 
-		NAObject *iter_object;
 		gtk_tree_model_get( model, &iter, IACTIONS_LIST_NAOBJECT_COLUMN, &iter_object, -1 );
 		if( iter_object == NA_OBJECT( action )){
 
 			if( na_action_get_profiles_count( action ) > 1 ){
 
-				GtkTreePath *path = gtk_tree_model_get_path( model, &iter );
+				path = gtk_tree_model_get_path( model, &iter );
 
 				if( gtk_tree_view_row_expanded( GTK_TREE_VIEW( treeview ), path )){
 					gtk_tree_view_collapse_row( GTK_TREE_VIEW( treeview ), path );
-					g_debug( "%s: action=%p collapsed", thisfn, action );
+					g_debug( "%s: action=%p collapsed", thisfn, ( void * ) action );
 
 				} else {
 					gtk_tree_view_expand_row( GTK_TREE_VIEW( treeview ), path, TRUE );
-					g_debug( "%s: action=%p expanded", thisfn, action );
+					g_debug( "%s: action=%p expanded", thisfn, ( void * ) action );
 				}
 
 				gtk_tree_path_free( path );
@@ -603,22 +633,32 @@ nact_iactions_list_toggle_collapse( NactWindow *window, const NAAction *action )
 void
 nact_iactions_list_update_selected( NactWindow *window, NAAction *action )
 {
-	GtkWidget *treeview = get_actions_list_widget( window );
-	GtkTreeSelection *selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( treeview ));
-
+	GtkWidget *treeview;
+	GtkTreeSelection *selection;
+	GList *listrows, *irow;
 	GtkTreeModel *tm_model;
+	GtkTreePath *path;
 	GtkTreeIter iter;
-	if( gtk_tree_selection_get_selected( selection, &tm_model, &iter )){
+	NAObject *object;
+	GtkTreeModelFilter *tmf_model;
+	GtkTreeStore *ts_model;
+	GtkTreeIter ts_iter;
+	GSList *profiles, *ip;
 
-		NAObject *object;
+	treeview = get_actions_list_widget( window );
+	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( treeview ));
+	listrows = gtk_tree_selection_get_selected_rows( selection, &tm_model );
+
+	for( irow = listrows ; irow ; irow = irow->next ){
+
+		path = ( GtkTreePath * ) irow->data;
+		gtk_tree_model_get_iter( tm_model, &iter, path );
 		gtk_tree_model_get( tm_model, &iter, IACTIONS_LIST_NAOBJECT_COLUMN, &object, -1 );
 		g_assert( object );
+		g_assert( NA_IS_OBJECT( object ));
 
-		GtkTreePath *path = gtk_tree_model_get_path( tm_model, &iter );
-
-		GtkTreeModelFilter *tmf_model = GTK_TREE_MODEL_FILTER( gtk_tree_view_get_model( GTK_TREE_VIEW( treeview )));
-		GtkTreeStore *ts_model = GTK_TREE_STORE( gtk_tree_model_filter_get_model( tmf_model ));
-		GtkTreeIter ts_iter;
+		tmf_model = GTK_TREE_MODEL_FILTER( gtk_tree_view_get_model( GTK_TREE_VIEW( treeview )));
+		ts_model = GTK_TREE_STORE( gtk_tree_model_filter_get_model( tmf_model ));
 		gtk_tree_model_get_iter( GTK_TREE_MODEL( ts_model ), &ts_iter, path );
 
 		if( NA_IS_ACTION( object )){
@@ -629,8 +669,7 @@ nact_iactions_list_update_selected( NactWindow *window, NAAction *action )
 		} else {
 			g_assert( NA_IS_ACTION_PROFILE( object ));
 			/*g_debug( "nact_iactions_list_update_selected: selected profile=%p", object );*/
-			GSList *profiles = na_action_get_profiles( action );
-			GSList *ip;
+			profiles = na_action_get_profiles( action );
 			for( ip = profiles ; ip ; ip = ip->next ){
 				if( object == NA_OBJECT( ip->data )){
 					setup_profile( treeview, ts_model, &ts_iter, NA_ACTION_PROFILE( ip->data ));
@@ -662,11 +701,14 @@ nact_iactions_list_set_edition_mode( NactWindow *window, gboolean edition )
 void
 nact_iactions_list_set_multiple_selection( NactWindow *window, gboolean multiple )
 {
+	GtkWidget *list;
+	GtkTreeSelection *selection;
+
 	g_assert( NACT_IS_IACTIONS_LIST( window ));
 	g_object_set_data( G_OBJECT( window ), ACCEPT_MULTIPLE_SELECTION, GINT_TO_POINTER( multiple ));
 
-	GtkWidget *list = get_actions_list_widget( window );
-	GtkTreeSelection *selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( list ));
+	list = get_actions_list_widget( window );
+	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( list ));
 	gtk_tree_selection_set_mode( selection, multiple ? GTK_SELECTION_MULTIPLE : GTK_SELECTION_SINGLE );
 }
 
@@ -691,11 +733,23 @@ nact_iactions_list_set_is_filling_list( NactWindow *window, gboolean is_filling
 	g_object_set_data( G_OBJECT( window ), IS_FILLING_LIST, GINT_TO_POINTER( is_filling ));
 }
 
+/**
+ * Does the TreeView implements Drag&Drop ?
+ */
+void
+nact_iactions_list_set_dnd_mode( NactWindow *window, gboolean have_dnd )
+{
+	g_assert( NACT_IS_IACTIONS_LIST( window ));
+	g_object_set_data( G_OBJECT( window ), HAVE_DND_MODE, GINT_TO_POINTER( have_dnd ));
+}
+
 static GSList *
 v_get_actions( NactWindow *window )
 {
+	NactIActionsList *instance;
+
 	g_assert( NACT_IS_IACTIONS_LIST( window ));
-	NactIActionsList *instance = NACT_IACTIONS_LIST( window );
+	instance = NACT_IACTIONS_LIST( window );
 
 	if( NACT_IACTIONS_LIST_GET_INTERFACE( instance )->get_actions ){
 		return( NACT_IACTIONS_LIST_GET_INTERFACE( instance )->get_actions( window ));
@@ -715,36 +769,18 @@ v_set_sorted_actions( NactWindow *window, GSList *actions )
 	}
 }*/
 
-static void
-v_on_selection_changed( GtkTreeSelection *selection, gpointer user_data )
-{
-	g_assert( NACT_IS_IACTIONS_LIST( user_data ));
-	g_assert( BASE_IS_WINDOW( user_data ));
-
-	NactIActionsList *instance = NACT_IACTIONS_LIST( user_data );
-
-	g_assert( NACT_IS_WINDOW( user_data ));
-	gboolean send_message = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( user_data ), SEND_SELECTION_CHANGED_MESSAGE ));
-	gboolean is_filling = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( user_data ), IS_FILLING_LIST ));
-
-	if( send_message || !is_filling ){
-		if( NACT_IACTIONS_LIST_GET_INTERFACE( instance )->on_selection_changed ){
-			NACT_IACTIONS_LIST_GET_INTERFACE( instance )->on_selection_changed( selection, user_data );
-		}
-	}
-}
-
 static gboolean
 v_on_button_press_event( GtkWidget *widget, GdkEventButton *event, gpointer user_data )
 {
 	/*static const gchar *thisfn = "nact_iactions_list_v_on_button_pres_event";
 	g_debug( "%s: widget=%p, event=%p, user_data=%p", thisfn, widget, event, user_data );*/
 
+	gboolean stop = FALSE;
+	NactIActionsList *instance;
+
 	g_assert( NACT_IS_IACTIONS_LIST( user_data ));
 	g_assert( NACT_IS_WINDOW( user_data ));
-
-	gboolean stop = FALSE;
-	NactIActionsList *instance = NACT_IACTIONS_LIST( user_data );
+	instance = NACT_IACTIONS_LIST( user_data );
 
 	if( NACT_IACTIONS_LIST_GET_INTERFACE( instance )->on_button_press_event ){
 		stop = NACT_IACTIONS_LIST_GET_INTERFACE( instance )->on_button_press_event( widget, event, user_data );
@@ -766,12 +802,14 @@ v_on_key_pressed_event( GtkWidget *widget, GdkEventKey *event, gpointer user_dat
 	/*static const gchar *thisfn = "nact_iactions_list_v_on_key_pressed_event";
 	g_debug( "%s: widget=%p, event=%p, user_data=%p", thisfn, widget, event, user_data );*/
 
+	gboolean stop = FALSE;
+	NactIActionsList *instance;
+
 	g_assert( NACT_IS_IACTIONS_LIST( user_data ));
 	g_assert( NACT_IS_WINDOW( user_data ));
 	g_assert( event->type == GDK_KEY_PRESS );
 
-	gboolean stop = FALSE;
-	NactIActionsList *instance = NACT_IACTIONS_LIST( user_data );
+	instance = NACT_IACTIONS_LIST( user_data );
 
 	if( NACT_IACTIONS_LIST_GET_INTERFACE( instance )->on_key_pressed_event ){
 		stop = NACT_IACTIONS_LIST_GET_INTERFACE( instance )->on_key_pressed_event( widget, event, user_data );
@@ -832,6 +870,24 @@ v_is_valid_profile( NactWindow *window, const NAActionProfile *profile )
 	return( FALSE );
 }
 
+static void
+on_selection_changed( GtkTreeSelection *selection, NactIActionsList *instance )
+{
+	gboolean send_message, is_filling;
+	GSList *selected_items;
+
+	send_message = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( instance ), SEND_SELECTION_CHANGED_MESSAGE ));
+	is_filling = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( instance ), IS_FILLING_LIST ));
+
+	selected_items = nact_iactions_list_get_selected_items( instance );
+
+	if( send_message || !is_filling ){
+		if( NACT_IACTIONS_LIST_GET_INTERFACE( instance )->on_selection_changed ){
+			NACT_IACTIONS_LIST_GET_INTERFACE( instance )->on_selection_changed( instance, selected_items );
+		}
+	}
+}
+
 /*
  * action modified: italic
  * action not saveable: red
@@ -840,13 +896,15 @@ static void
 display_label( GtkTreeViewColumn *column, GtkCellRenderer *cell, GtkTreeModel *model, GtkTreeIter *iter, NactWindow *window )
 {
 	NAObject *object;
+	gchar *label;
+	gboolean modified = FALSE;
+	gboolean valid = TRUE;
+
 	gtk_tree_model_get( model, iter, IACTIONS_LIST_NAOBJECT_COLUMN, &object, -1 );
 
 	if( object ){
 		g_assert( NA_IS_OBJECT( object ));
-		gchar *label = na_object_get_label( object );
-		gboolean modified = FALSE;
-		gboolean valid = TRUE;
+		label = na_object_get_label( object );
 
 		if( is_edition_mode( window )){
 			if( NA_IS_ACTION( object )){
@@ -883,6 +941,8 @@ setup_action( GtkWidget *treeview, GtkTreeStore *model, GtkTreeIter *iter, NAAct
 	static const gchar *thisfn = "nact_iactions_list_setup_action";
 	GtkStockItem item;
 	GdkPixbuf* icon = NULL;
+	gint width, height;
+	GError* error = NULL;
 
 	gchar *label = na_action_get_label( action );
 	gchar *iconname = na_action_get_icon( action );
@@ -897,9 +957,6 @@ setup_action( GtkWidget *treeview, GtkTreeStore *model, GtkTreeIter *iter, NAAct
 
 		} else if( g_file_test( iconname, G_FILE_TEST_EXISTS )
 			   && g_file_test( iconname, G_FILE_TEST_IS_REGULAR )){
-			gint width;
-			gint height;
-			GError* error = NULL;
 
 			gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &width, &height);
 			icon = gdk_pixbuf_new_from_file_at_size( iconname, width, height, &error );
@@ -926,43 +983,6 @@ setup_profile( GtkWidget *treeview, GtkTreeStore *model, GtkTreeIter *iter, NAAc
 	g_free( label );
 }
 
-static gint
-sort_actions_list( GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, NactWindow *window )
-{
-	g_debug( "sort_actions_list" );
-	gchar *labela, *labelb;
-
-	gtk_tree_model_get( model, a, IACTIONS_LIST_LABEL_COLUMN, &labela, -1 );
-	gtk_tree_model_get( model, b, IACTIONS_LIST_LABEL_COLUMN, &labelb, -1 );
-
-	gint ret = g_utf8_collate( labela, labelb );
-
-	g_free( labela );
-	g_free( labelb );
-
-	return( ret );
-}
-
-static gboolean
-filter_visible( GtkTreeModel *model, GtkTreeIter *iter, gpointer data )
-{
-	NAObject *object;
-	gtk_tree_model_get( model, iter, IACTIONS_LIST_NAOBJECT_COLUMN, &object, -1 );
-	/*g_debug( "nact_main_window_filer_visible: model=%p, iter=%p, data=%p, object=%p", model, iter, data, object );*/
-
-	if( object ){
-		if( NA_IS_ACTION( object )){
-			return( TRUE );
-		}
-
-		g_assert( NA_IS_ACTION_PROFILE( object ));
-		NAAction *action = na_action_profile_get_action( NA_ACTION_PROFILE( object ));
-		return( na_action_get_profiles_count( action ) > 1 );
-	}
-
-	return( FALSE );
-}
-
 static GtkWidget *
 get_actions_list_widget( NactWindow *window )
 {
@@ -979,11 +999,12 @@ get_expanded_rows( NactWindow *window )
 	GtkTreeIter iter;
 	gboolean iterok = gtk_tree_model_get_iter_first( model, &iter );
 	NAObject *object;
+	GtkTreePath *path;
 
 	while( iterok ){
 		gtk_tree_model_get( model, &iter, IACTIONS_LIST_NAOBJECT_COLUMN, &object, -1 );
 
-		GtkTreePath *path = gtk_tree_model_get_path( model, &iter );
+		path = gtk_tree_model_get_path( model, &iter );
 
 		if( gtk_tree_view_row_expanded( GTK_TREE_VIEW( treeview ), path )){
 			expanded = g_slist_prepend( expanded, object );
@@ -1006,15 +1027,16 @@ expand_rows( NactWindow *window, GSList *expanded )
 	GtkTreeIter iter;
 	gboolean iterok = gtk_tree_model_get_iter_first( model, &iter );
 	NAObject *object;
+	GSList *is;
+	GtkTreePath *path;
 
 	while( iterok ){
 		gtk_tree_model_get( model, &iter, IACTIONS_LIST_NAOBJECT_COLUMN, &object, -1 );
 
-		GSList *is;
 		for( is = expanded ; is ; is = is->next ){
 			if( object == NA_OBJECT( is->data )){
 
-				GtkTreePath *path = gtk_tree_model_get_path( model, &iter );
+				path = gtk_tree_model_get_path( model, &iter );
 				gtk_tree_view_expand_row( GTK_TREE_VIEW( treeview ), path, TRUE );
 				gtk_tree_path_free( path );
 
